#!/bin/bash

printf "\033c" # Clear screen

# Get OS type
case "$OSTYPE" in
  cygwin*)
    OS=Windows
  ;;
  linux*)
    OS=Linux
  ;;
  darwin*)
    OS=OSX
  ;;
  *)
    echo "Unable to identify OS type $OSTYPE..."
    echo "Press ENTER key to continue..."
    read
    exit
  ;;
esac

# Get path variables
if [ "$OS" != "Windows" ]; then
  ROOTPATH=`pwd`
  AWK=awk
else
  ROOTPATH=`cygpath -m "$1"`
  AWK=gawk
fi
cd $ROOTPATH

UPDATE_ZIP=$ROOTPATH/update.zip
TMP_DIR=$ROOTPATH/output
UPDATE_APP=$TMP_DIR/UPDATE.APP
ZIP_TOOL=$ROOTPATH/../tools/7za.exe
SPLIT_TOOL=$ROOTPATH/../tools/splitupdate.exe
RED='\033[0;31m' #Red
GREEN='\033[0;32m' #Green
NC='\033[0m' # No Color

CHECK_FILE=1

### Support function

function pause {
  if [ "$1" != "" ]; then
    echo $1
  fi
  echo "Press ENTER key to continue..."
  read
}

function getkeys {
  read keyinput
  echo $keyinput
}

function clearkeys {
  while read -r -t 0; do read -r; done
}

function isnum {
  if [ $(echo "$1" | grep -c '^[0-9]\+$') = 1 ]; then
    echo 1
  else
    echo 0
  fi
}

function format_str {
  strlen=${#1}
  count=$2
  remain=$(( count - strlen ))
  echo -n "$1"
  printf '%*s' "$remain"
}

function unzipfile {
  if [ ! -d "$ROOTPATH/output" ]; then
  mkdir $ROOTPATH/output
  fi
  $ZIP_TOOL e $UPDATE_ZIP UPDATE.APP -o$TMP_DIR > /dev/null 2>&1
  if [ -f "$UPDATE_APP" ]; then
    echo 0
  else
    echo 1
  fi
}

#######################################################

### Check rootpath
if [ "`echo $ROOTPATH | grep ' '`" != "" ]; then
  pause "This script does not support directory with space."
  exit
fi

if [ ${#ROOTPATH} -gt 200 ]; then
  pause "Path is too long, please extract script package in shorter path."
  exit
fi

echo 

### Check file existence
if [ "$CHECK_FILE" -eq "1" ]; then
  if [ ! -f "$UPDATE_ZIP" ]; then
    pause "No update.zip file found!"
    exit 1
  fi
fi

echo "Extracting UPDATE.APP, FULLOTA takes a bit. Please wait"
if [ "$(unzipfile $UPDATE_ZIP)" != "0" ]; then
  pause "Extract update.zip failed!"
  exit 2
fi

echo "Extracting XLOADER from UPDATE.APP"
$SPLIT_TOOL $TMP_DIR/UPDATE.APP XLOADER > /dev/null 2>&1
if [ ! -f "$TMP_DIR/XLOADER.img" ]; then
  pause "Extract XLOADER failed"
  exit 2
fi

echo "Checking XLOADER"
xxd -ps -s 0x1a8 -l 1 $TMP_DIR/XLOADER.img | grep 01 > /dev/null && echo -e "${GREEN}01 - Safe when going 01 to 01 or 01 to 02${NC}" || echo -e "${RED}02 - Safe when going 02 to 02${NC}"
echo -e "\nYou can flash 02 XLOADER ontop of 01 XLOADER, but you can't flash 01 XLOADER ontop of 02 XLOADER.\n01 to 01, 01 to 02 and 02 to 02 is safe. 02 to 01 is not!"
echo -e "\nCleaning up"
rm -rf $ROOTPATH/output
pause "All done!"
